@@ -1,5 +1,5 @@
 /*
-	htpdate v1.1.1
+	htpdate v1.1.3
 
 	Eddy Vervest <eddy@vervest.org>
 	http://www.vervest.org/htp
@@ -52,7 +52,7 @@
 #include <pwd.h>
 #include <grp.h>
 
-#define VERSION 				"1.1.1"
+#define VERSION 				"1.1.3"
 #define	MAX_HTTP_HOSTS			15				/* 16 web servers */
 #define	DEFAULT_HTTP_PORT		"80"
 #define	DEFAULT_PROXY_PORT		"8080"
@@ -73,7 +73,7 @@
 /* By default we turn off "debug" and "log" mode  */
 static int		debug = 0;
 static int		logmode = 0;
-static time_t	gmtoffset;
+static time_t	gmtoffset = 0;
 
 
 /* Insertion sort is more efficient (and smaller) than qsort for small lists */
@@ -141,6 +141,21 @@ static void printlog( int is_error, char *format, ... ) {
 }
 
 
+/* Drop or elevate privileges */
+static void swuid( int id ) {
+	if ( seteuid( id ) ) {
+		printlog( 1, "seteuid() %i", id );
+		exit(1);
+	}
+}
+static void swgid( int id ) {
+	if ( setegid( id ) ) {
+		printlog( 1, "setegid() %i", id );
+		exit(1);
+	}
+}
+
+
 static long getHTTPdate( char *host, char *port, char *proxy, char *proxyport, char *httpversion, int ipversion, int when ) {
 	int					server_s;
 	int					rc;
@@ -315,12 +330,8 @@ static int setclock( double timedelta, int setmode ) {
 		printlog( 0, "Adjusting %.3f seconds", timedelta );
 
 		/* Become root */
-		if ( seteuid(0) ) {
-			printlog( 1, "seteuid()" );
-			exit(1);
-		} else {
-			return( adjtime(&timeofday, NULL) );
-		}
+		swuid(0);
+		return( adjtime(&timeofday, NULL) );
 
 	case 2:					/* Set time */
 		printlog( 0, "Setting %.3f seconds", timedelta );
@@ -334,12 +345,8 @@ static int setclock( double timedelta, int setmode ) {
 		printlog( 0, "Set: %s", asctime(localtime(&timeofday.tv_sec)) );
 
 		/* Become root */
-		if ( seteuid(0) ) {
-			printlog( 1, "seteuid()" );
-			exit(1);
-		} else {
-			return( settimeofday(&timeofday, NULL) );
-		}
+		swuid(0);
+		return( settimeofday(&timeofday, NULL) );
 
 	case 3:					/* Set frequency, but first an adjust */
 		return( setclock( timedelta, 1 ) );
@@ -373,12 +380,8 @@ static int htpdate_adjtimex( double drift ) {
 	tmx.modes = MOD_FREQUENCY;
 
 	/* Become root */
-	if ( seteuid(0) ) {
-		printlog( 1, "seteuid()" );
-		exit(1);
-	} else {
-		return( ntp_adjtime(&tmx) );
-	}
+	swuid(0);
+	return( ntp_adjtime(&tmx) );
 
 }
 
@@ -638,11 +641,10 @@ int main( int argc, char *argv[] ) {
 	}
 
 	/* Now we are root, we drop the privileges (if specified) */
-	if ( sw_uid ) seteuid( sw_uid );
-	if ( sw_gid ) setegid( sw_gid );
+	if ( sw_gid ) swgid( sw_gid );
+	if ( sw_uid ) swuid( sw_uid );
 
     /* Calculate GMT offset from local timezone */
-    time(&gmtoffset);
     gmtoffset -= mktime(gmtime(&gmtoffset));
 
 	/* In case we have more than one web server defined, we
@@ -770,7 +772,7 @@ int main( int argc, char *argv[] ) {
 					printlog( 1, "Time change failed" );
 
 			/* Drop root privileges again */
-			if ( sw_uid ) seteuid( sw_uid );
+			swuid( sw_uid );
 
 			if ( daemonize ) {
 				if ( starttime ) {
@@ -787,7 +789,7 @@ int main( int argc, char *argv[] ) {
 							printlog( 1, "Frequency change failed" );
 
 						/* Drop root privileges again */
-						if ( sw_uid ) seteuid( sw_uid );
+						swuid( sw_uid );
 					}
 				} else {
 					starttime = time(NULL);
